/*------------------------------------------------------------------------------
* Copyright (C) 2009 Isidor Zeuner
* 
* Distributable under the terms of either the Apache License (Version 2.0) or 
* the GNU Lesser General Public License, as specified in the COPYING file.
------------------------------------------------------------------------------*/
#ifndef CLUCENE_UTIL_BYTEINPUTSTREAM_H
#define CLUCENE_UTIL_BYTEINPUTSTREAM_H

#include "arrayinputstream.h"

CL_NS_DEF(util)

typedef ArrayInputStream<signed char> ByteInputStream;

CL_NS_END
#endif
